# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class UpsertResourceRecordRequest(BaseModel):
    """
    UpsertResourceRecordRequest
    """
    deployment_scope:  StrictStr = Field(...,alias="deploymentScope") 
    deployment_code:  StrictStr = Field(...,alias="deploymentCode") 
    resource_id:  StrictStr = Field(...,alias="resourceId") 
    format:  StrictStr = Field(...,alias="format") 
    resource_type:  StrictStr = Field(...,alias="resourceType") 
    resource_state: Optional[Any] = Field(alias="resourceState")
    dependencies: List[StrictStr]
    tracking_state: Optional[Any] = Field(default=None, alias="trackingState")
    __properties = ["deploymentScope", "deploymentCode", "resourceId", "format", "resourceType", "resourceState", "dependencies", "trackingState"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpsertResourceRecordRequest:
        """Create an instance of UpsertResourceRecordRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if resource_state (nullable) is None
        # and __fields_set__ contains the field
        if self.resource_state is None and "resource_state" in self.__fields_set__:
            _dict['resourceState'] = None

        # set to None if tracking_state (nullable) is None
        # and __fields_set__ contains the field
        if self.tracking_state is None and "tracking_state" in self.__fields_set__:
            _dict['trackingState'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpsertResourceRecordRequest:
        """Create an instance of UpsertResourceRecordRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpsertResourceRecordRequest.parse_obj(obj)

        _obj = UpsertResourceRecordRequest.parse_obj({
            "deployment_scope": obj.get("deploymentScope"),
            "deployment_code": obj.get("deploymentCode"),
            "resource_id": obj.get("resourceId"),
            "format": obj.get("format"),
            "resource_type": obj.get("resourceType"),
            "resource_state": obj.get("resourceState"),
            "dependencies": obj.get("dependencies"),
            "tracking_state": obj.get("trackingState")
        })
        return _obj

UpsertResourceRecordRequest.update_forward_refs()
