# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.resource_id import ResourceId

class SweepBlocksRequest(BaseModel):
    """
    A request to sweep specified blocks.  # noqa: E501
    """
    block_ids: Dict[str, ResourceId] = Field(description="A dictionary mapping ephemeral identifiers, which live as long as the request, to specific blocks to sweep.", alias="blockIds")
    latest_allowable_modification_time:  StrictStr = Field(...,alias="latestAllowableModificationTime", description="Timestamp or cut label which the  block or related entities must not have been updated after.") 
    __properties = ["blockIds", "latestAllowableModificationTime"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SweepBlocksRequest:
        """Create an instance of SweepBlocksRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in block_ids (dict)
        _field_dict = {}
        if self.block_ids:
            for _key in self.block_ids:
                if self.block_ids[_key]:
                    _field_dict[_key] = self.block_ids[_key].to_dict()
            _dict['blockIds'] = _field_dict
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SweepBlocksRequest:
        """Create an instance of SweepBlocksRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SweepBlocksRequest.parse_obj(obj)

        _obj = SweepBlocksRequest.parse_obj({
            "block_ids": dict(
                (_k, ResourceId.from_dict(_v))
                for _k, _v in obj.get("blockIds").items()
            )
            if obj.get("blockIds") is not None
            else None,
            "latest_allowable_modification_time": obj.get("latestAllowableModificationTime")
        })
        return _obj

SweepBlocksRequest.update_forward_refs()
