# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.model_property import ModelProperty
from lusid.models.version import Version

class EstimateVariant(BaseModel):
    """
    EstimateVariant
    """
    variant:  Optional[StrictStr] = Field(None,alias="variant", description="The Variant of the Calendar Entry. Together with the valuation point code marks the unique branch for the NavType.") 
    display_name:  StrictStr = Field(...,alias="displayName", description="The name of the Fund Calendar entry.") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="A description for the Fund Calendar entry.") 
    as_at: datetime = Field(description="The asAt datetime for the Calendar Entry.", alias="asAt")
    holdings_as_at_override: Optional[datetime] = Field(default=None, description="The optional AsAt Override to use for building holdings in the Valuation Point. Defaults to Latest.", alias="holdingsAsAtOverride")
    valuations_as_at_override: Optional[datetime] = Field(default=None, description="The optional AsAt Override to use for performing valuations in the Valuation Point. Defaults to Latest.", alias="valuationsAsAtOverride")
    properties: Optional[Dict[str, ModelProperty]] = Field(default=None, description="The properties for the Calendar Entry. These will be from the 'ClosedPeriod' domain.")
    version: Version
    __properties = ["variant", "displayName", "description", "asAt", "holdingsAsAtOverride", "valuationsAsAtOverride", "properties", "version"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EstimateVariant:
        """Create an instance of EstimateVariant from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # set to None if variant (nullable) is None
        # and __fields_set__ contains the field
        if self.variant is None and "variant" in self.__fields_set__:
            _dict['variant'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if holdings_as_at_override (nullable) is None
        # and __fields_set__ contains the field
        if self.holdings_as_at_override is None and "holdings_as_at_override" in self.__fields_set__:
            _dict['holdingsAsAtOverride'] = None

        # set to None if valuations_as_at_override (nullable) is None
        # and __fields_set__ contains the field
        if self.valuations_as_at_override is None and "valuations_as_at_override" in self.__fields_set__:
            _dict['valuationsAsAtOverride'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> EstimateVariant:
        """Create an instance of EstimateVariant from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return EstimateVariant.parse_obj(obj)

        _obj = EstimateVariant.parse_obj({
            "variant": obj.get("variant"),
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "as_at": obj.get("asAt"),
            "holdings_as_at_override": obj.get("holdingsAsAtOverride"),
            "valuations_as_at_override": obj.get("valuationsAsAtOverride"),
            "properties": dict(
                (_k, ModelProperty.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None,
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None
        })
        return _obj

EstimateVariant.update_forward_refs()
